#ifndef _TUNOUTDEVICE_H__
#define _TUNOUTDEVICE_H__

#include <omnetpp.h>
#include "INETDefs.h"
#include "ctunoutscheduler.h"
#include "PacketParser.h"

class InterfaceEntry;

/**
 * needed for computing checksums
 *
 * \param buf The buffer containing the data to be checksummed
 * \param nbytes The length of the buffer in bytes
 * \return the Checksum
 */
u_short cksum(uint16_t *buf, int nbytes)
{
    register unsigned long  sum;
    u_short                 oddbyte;

    sum = 0;
    while (nbytes > 1) {
        sum += *buf++;
        nbytes -= 2;
    }

    if (nbytes == 1) {
        oddbyte = 0;
        *((u_char *) &oddbyte) = *(u_char *) buf;
        sum += oddbyte;
    }

    sum = (sum >> 16) + (sum & 0xffff);
    sum += (sum >> 16);

    return ~sum;
}

/**
 * TunOutDevice is a pseudo interface that allows communcation with the real world
 * through the cTunOutScheduler
 *
 * WARNING: This does ONLY work with the combination IPv4|UDP|OverlayMessage
 */
class INET_API TunOutDevice : public cSimpleModule
{
protected:
#ifdef _MAX_SPEED

    int gateIndexNetwOut;
#endif

    InterfaceEntry *interfaceEntry;  // points into InterfaceTable
    unsigned int mtu;
#define BUFFERZITE mtu + 4 // 4 bytes for packet information

    // statistics
    long numSent;
    long numSendError;
    long numRcvdOK;
    long numRcvError;

    cMessage* packetNotification; // used by cTunOutScheduler to nitify about new packets
    cTunOutScheduler::PacketBuffer packetBuffer; // recieved packets are stored here
    cTunOutScheduler* scheduler;
    PacketParser* parser;

    /** Register the interface in the interface table of the parent
     *
     * \return A pointer to the Interface entry
     */
    InterfaceEntry *registerInterface();

    /** Send a message to the tun device.
     *  
     * \param msg A pointer to the message to be send
     */
    void transmitToTun(cMessage *msg);
    void updateDisplayString();

    /**
     * Converts an IP datagram to a data block for sending it to the tun device
     *
     * \param msg A pointer to the message to be converted
     * \param length A pointer to an int that will hold the length of the converted data
     * \return A pointer to the converted data
     */
    virtual char* encapsulate(cMessage *msg, unsigned int* length);

    /**
     * Parses data received from the tun device and converts it into a cMessage
     *
     * \param buf A pointer to the data to be parsed
     * \return The parsed message
     */
    virtual cMessage *decapsulate(char* buf);

public:
    Module_Class_Members(TunOutDevice, cSimpleModule, 0);

    virtual int numInitStages() const
    {
        return 4;
    }

    /** Initialization of the module.
     * Registers the device at the scheduler and searches for the appropriate payload-parser
     * Will be called automatically at startup
     */
    virtual void initialize(int stage);

    /**
     * The "main loop". Every message that is received or send is handled by this method
     */
    virtual void handleMessage(cMessage *msg);
};

#endif


