#!/usr/bin/perl

# Postprocess OverSim output files (.sca or .vec)

use Statistics::Descriptive;

# Output for a single run from a .sca file
sub printScaStats {
  print "\n======================================================================================\n";
  print "                    $run";
  print "======================================================================================\n";
  foreach $key (sort keys(%entry)){
    print "$key: " . $entry{$key}->mean();
    print " (min: " . $entry{$key}->min();
    print ", max: " . $entry{$key}->max() . ")\n";
  }
}

# Processing of a .sca file
sub processSca {
  while(<>){
    if (/^run/) { # New run starts
      $newrun = $_;
      if (keys %entry) {&printScaStats();} # if last run not empty, print stats
      $run = $newrun;
      %entry = ();
    } else {
      s/\[\d*\]/\[\]/g; # Delete digits in square brackets
      # Parse lines: 'scalar "<Module>" "<Param>" <Value>'
      /scalar\s*"([^"]*)"\s*("[^"]*")\s*(\d*)/;
      $name = "$1 $2";
      if (! defined $entry{$name} ) { $entry{$name} = Statistics::Descriptive::Full->new(); }
      $entry{$name}->add_data($3);
    }
  }
  &printScaStats();
}

# Processing of a .vec file
sub processVec {
  while(<>){
    if (/^vector/) {
      s/\[[0-9]*\]/\[\]/g; # Delete digits in square brackets
      /(\d*)  "(.*)"  (".*")  (\d*)\n$/; # Parse vector definition
      $vectorname[$1] = "$2 $3";

    } elsif (/^([0-9]*)\t([0-9.]*)\t([0-9.]*)\n$/) {
      if (! defined $entry{$vectorname[$1]} ) {
	$entry{$vectorname[$1]} = Statistics::Descriptive::Full->new();
      }
      $entry{$vectorname[$1]}->add_data($3);
    }
  }

  foreach $key (sort keys(%entry)){
    print "$key: " . $entry{$key}->mean();
    print " (min: " . $entry{$key}->min();
    print ", max: " . $entry{$key}->max();
    print ", 10p: " . $entry{$key}->percentile(10);
    print ", 90p: " . $entry{$key}->percentile(90) . ")\n";
  }
}

# Check whether input is .sca or .vec
$line = <>;
$_ = $line;
if (/^run/) {
  # .sca
  $run = $line;
  &processSca();

} elsif (/^vector/) {
  # .vec
  s/\[[0-9]*\]/\[\]/g; # Delete digits in square brackets
  /(\d*)  "(.*)"  (".*")  (\d*)\n$/; # Parse vector definition
  $vectorname[$1] = "$2 $3";
  &processVec();

} else {
  die "File type not recognized!\n";
}

